@@ -21,7 +21,7 @@ import zlib
 
 from PyInstaller.building.utils import get_code_object, strip_paths_in_code
 from PyInstaller.compat import BYTECODE_MAGIC, is_win, strict_collect_mode
-from PyInstaller.loader.pyimod01_archive import PYZ_ITEM_DATA, PYZ_ITEM_MODULE, PYZ_ITEM_NSPKG, PYZ_ITEM_PKG
+from PyInstaller.loader.pyimod01_archive import PYZ_ITEM_MODULE, PYZ_ITEM_NSPKG, PYZ_ITEM_PKG
 
 
 class ZlibArchiveWriter:
@@ -33,19 +33,16 @@ class ZlibArchiveWriter:
     _HEADER_LENGTH = 12 + 5
     _COMPRESSION_LEVEL = 6  # zlib compression level
 
-    def __init__(self, filename, entries, code_dict=None, cipher=None):
+    def __init__(self, filename, entries, code_dict=None):
         """
         filename
             Target filename of the archive.
         entries
             An iterable containing entries in the form of tuples: (name, src_path, typecode), where `name` is the name
             under which the resource is stored (e.g., python module name, without suffix), `src_path` is name of the
-            file from which the resource is read, and `typecode` is the Analysis-level TOC typecode (`PYMODULE` or
-            `DATA`).
+            file from which the resource is read, and `typecode` is the Analysis-level TOC typecode (`PYMODULE`).
         code_dict
             Optional code dictionary containing code objects for analyzed/collected python modules.
-        cipher
-            Optional `Cipher` object for bytecode encryption.
         """
         code_dict = code_dict or {}
 
@@ -56,7 +53,7 @@ class ZlibArchiveWriter:
             # Write entries' data and collect TOC entries
             toc = []
             for entry in entries:
-                toc_entry = self._write_entry(fp, entry, code_dict, cipher)
+                toc_entry = self._write_entry(fp, entry, code_dict)
                 toc.append(toc_entry)
 
             # Write TOC
@@ -68,42 +65,31 @@ class ZlibArchiveWriter:
             #  - PYZ magic pattern (4 bytes)
             #  - python bytecode magic pattern (4 bytes)
             #  - TOC offset (32-bit int, 4 bytes)
-            #  - encryption flag (1 byte)
             #  - 4 unused bytes
             fp.seek(0, os.SEEK_SET)
 
             fp.write(self._PYZ_MAGIC_PATTERN)
             fp.write(BYTECODE_MAGIC)
             fp.write(struct.pack('!i', toc_offset))
-            fp.write(struct.pack('!B', cipher is not None))
 
     @classmethod
-    def _write_entry(cls, fp, entry, code_dict, cipher):
+    def _write_entry(cls, fp, entry, code_dict):
         name, src_path, typecode = entry
+        assert typecode == 'PYMODULE'
 
-        if typecode == 'PYMODULE':
-            typecode = PYZ_ITEM_MODULE
-            if src_path in ('-', None):
-                # This is a NamespacePackage, modulegraph marks them by using the filename '-'. (But wants to use None,
-                # so check for None, too, to be forward-compatible.)
-                typecode = PYZ_ITEM_NSPKG
-            else:
-                src_basename, _ = os.path.splitext(os.path.basename(src_path))
-                if src_basename == '__init__':
-                    typecode = PYZ_ITEM_PKG
-            data = marshal.dumps(code_dict[name])
+        typecode = PYZ_ITEM_MODULE
+        if src_path in ('-', None):
+            # This is a NamespacePackage, modulegraph marks them by using the filename '-'. (But wants to use None,
+            # so check for None, too, to be forward-compatible.)
+            typecode = PYZ_ITEM_NSPKG
         else:
-            # Any data files, that might be required by pkg_resources.
-            typecode = PYZ_ITEM_DATA
-            with open(src_path, 'rb') as fh:
-                data = fh.read()
-            # No need to use forward slash as path-separator here since pkg_resources on Windows uses back slash as
-            # path-separator.
+            src_basename, _ = os.path.splitext(os.path.basename(src_path))
+            if src_basename == '__init__':
+                typecode = PYZ_ITEM_PKG
+        data = marshal.dumps(code_dict[name])
 
         # First compress, then encrypt.
         obj = zlib.compress(data, cls._COMPRESSION_LEVEL)
-        if cipher:
-            obj = cipher.encrypt(obj)
 
         # Create TOC entry
         toc_entry = (name, (typecode, fp.tell(), len(obj)))
@@ -180,6 +166,11 @@ class CArchiveWriter:
     def _write_entry(self, fp, entry):
         dest_name, src_name, compress, typecode = entry
 
+        # Write OPTION entries as-is, without normalizing them. This also exempts them from duplication check,
+        # allowing them to be specified multiple times.
+        if typecode == 'o':
+            return self._write_blob(fp, b"", dest_name, typecode)
+
         # Ensure forward slashes in paths are on Windows converted to back slashes '\\', as on Windows the bootloader
         # works only with back slashes.
         dest_name = os.path.normpath(dest_name)
@@ -192,8 +183,8 @@ class CArchiveWriter:
         # (a file with same destination name, subject to OS case normalization rules).
         if strict_collect_mode:
             normalized_dest = None
-            if type in ('o', 's', 'm', 'M'):
-                # Exempt options, python source script, and modules from the check
+            if typecode in ('s', 'm', 'M'):
+                # Exempt python source scripts and modules from the check.
                 pass
             else:
                 # Everything else; normalize the case
@@ -206,9 +197,7 @@ class CArchiveWriter:
                     )
                 self._collected_names.add(normalized_dest)
 
-        if typecode == 'o':
-            return self._write_blob(fp, b"", dest_name, typecode)
-        elif typecode == 'd':
+        if typecode == 'd':
             # Dependency; merge src_name (= reference path prefix) and dest_name (= name) into single-string format that
             # is parsed by bootloader.
             return self._write_blob(fp, b"", f"{src_name}:{dest_name}", typecode)
@@ -229,6 +218,10 @@ class CArchiveWriter:
             # These module entries are loaded and executed within the bootloader, which requires only the code
             # object, without the PYC header.
             return self._write_blob(fp, marshal.dumps(code), dest_name, typecode, compress=compress)
+        elif typecode == 'n':
+            # Symbolic link; store target name (as NULL-terminated string)
+            data = src_name.encode('utf-8') + b'\x00'
+            return self._write_blob(fp, data, dest_name, typecode, compress=compress)
         else:
             return self._write_file(fp, src_name, dest_name, typecode, compress=compress)
 
@@ -349,6 +342,18 @@ class SplashWriter:
         :param str script: The tcl/tk script to execute to create the screen.
         """
 
+        # Ensure forward slashes in dependency names are on Windows converted to back slashes '\\', as on Windows the
+        # bootloader works only with back slashes.
+        def _normalize_filename(filename):
+            filename = os.path.normpath(filename)
+            if is_win and os.path.sep == '/':
+                # When building under MSYS, the above path normalization uses Unix-style separators, so replace them
+                # manually.
+                filename = filename.replace(os.path.sep, '\\')
+            return filename
+
+        name_list = [_normalize_filename(name) for name in name_list]
+
         with open(filename, "wb") as fp:
             # Reserve space for the header.
             fp.write(b'\0' * self._HEADER_LENGTH)
