@@ -12,13 +12,15 @@ import io
 import os
 import re
 import struct
+import pathlib
 
 from PyInstaller import log as logging
 from PyInstaller.archive.writers import SplashWriter
 from PyInstaller.building import splash_templates
-from PyInstaller.building.datastruct import TOC, Target
+from PyInstaller.building.datastruct import Target
 from PyInstaller.building.utils import _check_guts_eq, _check_guts_toc, misc
-from PyInstaller.compat import is_darwin, is_win, is_cygwin
+from PyInstaller.compat import is_darwin
+from PyInstaller.depend import bindepend
 from PyInstaller.utils.hooks import tcl_tk as tcltk_utils
 
 try:
@@ -50,7 +52,7 @@ class Splash(Target):
     """
     Bundles the required resources for the splash screen into a file, which will be included in the CArchive.
 
-    A Splash has two outputs, one is itself and one is sored in splash.binaries. Both need to be passed to other
+    A Splash has two outputs, one is itself and one is stored in splash.binaries. Both need to be passed to other
     build targets in order to enable the splash screen.
     """
     def __init__(self, image_file, binaries, datas, **kwargs):
@@ -61,20 +63,20 @@ class Splash(Target):
             .. note:: If a different file format is supplied and PIL (Pillow) is installed, the file will be converted
                 automatically.
 
-            .. note:: *Windows*: Due to the implementation, the color Magenta/ RGB(255, 0, 255) must not be used in the
-                image or text.
+            .. note:: *Windows*: The color ``'magenta'`` / ``'#ff00ff'`` must not be used in the image or text, as it is
+                used by splash screen to indicate transparent areas. Use a similar color (e.g., ``'#ff00fe'``) instead.
 
             .. note:: If PIL (Pillow) is installed and the image is bigger than max_img_size, the image will be resized
                 to fit into the specified area.
-        :param TOC binaries:
-            The TOC of binaries the Analysis build target found. This TOC includes all extensionmodules and their
-            dependencies. This is required to figure out, if the users program uses tkinter.
-        :param TOC datas:
-            The TOC of data the Analysis build target found. This TOC includes all data-file dependencies of the
+        :param list binaries:
+            The TOC list of binaries the Analysis build target found. This TOC includes all extension modules and their
+            binary dependencies. This is required to determine whether the user's program uses `tkinter`.
+        :param list datas:
+            The TOC list of data the Analysis build target found. This TOC includes all data-file dependencies of the
             modules. This is required to check if all splash screen requirements can be bundled.
 
         :keyword text_pos:
-            An optional 2x integer tuple that represents the origin of the text on the splash screen image. The
+            An optional two-integer tuple that represents the origin of the text on the splash screen image. The
             origin of the text is its lower left corner. A unit in the respective coordinate system is a pixel of the
             image, its origin lies in the top left corner of the image. This parameter also acts like a switch for
             the text feature. If omitted, no text will be displayed on the splash screen. This text will be used to
@@ -88,11 +90,12 @@ class Splash(Target):
             An optional name of a font for the text. This font must be installed on the user system, otherwise the
             system default font is used. If this parameter is omitted, the default font is also used.
         :keyword text_color:
-            An optional color for the text. Either RGB HTML notation or color names are supported. Default: black
-            (Windows: Due to a implementation issue the color magenta/ rgb(255, 0, 255) is forbidden)
+            An optional color for the text. HTML color codes (``'#40e0d0'``) and color names (``'turquoise'``) are
+            supported. Default: ``'black'``
+            (Windows: the color ``'magenta'`` / ``'#ff00ff'`` is used to indicate transparency, and should not be used)
         :type text_color: str
         :keyword text_default:
-            The default text which will be displayed before the extraction starts. Default: "Initializing"
+            The default text which will be displayed before the extraction starts. Default: ``"Initializing"``
         :type text_default: str
         :keyword full_tk:
             By default Splash bundles only the necessary files for the splash screen (some tk components). This
@@ -102,10 +105,10 @@ class Splash(Target):
         :type full_tk: bool
         :keyword minify_script:
             The splash screen is created by executing an Tcl/Tk script. This option enables minimizing the script,
-            meaning removing all non essential parts from the script. Default: True
+            meaning removing all non essential parts from the script. Default: ``True``
         :keyword rundir:
             The folder name in which tcl/tk will be extracted at runtime. There should be no matching folder in your
-            application to avoid conflicts. Default:  ``__splash``
+            application to avoid conflicts. Default:  ``'__splash'``
         :type rundir: str
         :keyword name:
             An optional alternative filename for the .res file. If not specified, a name is generated.
@@ -117,12 +120,12 @@ class Splash(Target):
         :keyword max_img_size:
             Maximum size of the splash screen image as a tuple. If the supplied image exceeds this limit, it will be
             resized to fit the maximum width (to keep the original aspect ratio). This option can be disabled by
-            setting it to None. Default: (760, 480)
+            setting it to None. Default: ``(760, 480)``
         :type max_img_size: Tuple[int, int]
         :keyword always_on_top:
             Force the splashscreen to be always on top of other windows. If disabled, other windows (e.g., from other
             applications) can cover the splash screen by user bringing them to front. This might be useful for
-            frozen applications with long startup times. Default: True
+            frozen applications with long startup times. Default: ``True``
         :type always_on_top: bool
         """
         from ..config import CONF
@@ -180,12 +183,13 @@ class Splash(Target):
             self._tkinter_file = self._tkinter_module.__file__
         except ModuleNotFoundError:
             raise SystemExit(
-                "You platform does not support the splash screen feature, since tkinter is not installed. Please "
+                "Your platform does not support the splash screen feature, since tkinter is not installed. Please "
                 "install tkinter and try again."
             )
 
         # Calculated / analysed values
-        self.uses_tkinter = self._uses_tkinter(binaries)
+        self.uses_tkinter = self._uses_tkinter(self._tkinter_file, binaries)
+        logger.debug("Program uses tkinter: %r", self.uses_tkinter)
         self.script = self.generate_script()
         self.tcl_lib, self.tk_lib = tcltk_utils.find_tcl_tk_shared_libs(self._tkinter_file)
         if is_darwin:
@@ -197,65 +201,52 @@ class Splash(Target):
         # Check if tcl/tk was found
         assert all(self.tcl_lib)
         assert all(self.tk_lib)
-        logger.debug("Use Tcl Library from %s and Tk From %s" % (self.tcl_lib, self.tk_lib))
+        logger.debug("Use Tcl Library from %s and Tk From %s", self.tcl_lib, self.tk_lib)
         self.splash_requirements = set([self.tcl_lib[0], self.tk_lib[0]] + splash_requirements)
 
         logger.info("Collect tcl/tk binaries for the splash screen")
         tcltk_tree = tcltk_utils.collect_tcl_tk_files(self._tkinter_file)
         if self.full_tk:
             # The user wants a full copy of tk, so make all tk files a requirement.
-            self.splash_requirements.update(toc[0] for toc in tcltk_tree)
+            self.splash_requirements.update(entry[0] for entry in tcltk_tree)
 
-        self.binaries = TOC()
+        # Scan for binary dependencies of the Tcl/Tk shared libraries, and add them to `binaries` TOC list (which
+        # should really be called `dependencies` as it is not limited to binaries. But it is too late now, and
+        # existing spec files depend on this naming). We specify these binary dependencies (which include the
+        # Tcl and Tk shared libaries themselves) even if the user's program uses tkinter and they would be collected
+        # anyway; let the collection mechanism deal with potential duplicates.
+        tcltk_libs = [(dest_name, src_name, 'BINARY') for dest_name, src_name in (self.tcl_lib, self.tk_lib)]
+        self.binaries = bindepend.binary_dependency_analysis(tcltk_libs)
+
+        # Put all shared library dependencies in `splash_requirements`, so they are made available in onefile mode.
+        self.splash_requirements.update(entry[0] for entry in self.binaries)
+
+        # If the user's program does not use tkinter, add resources from Tcl/Tk tree to the dependencies list.
+        # Do so only for the resources that are part of splash requirements.
         if not self.uses_tkinter:
-            # The user's script does not use tkinter, so we need to provide a TOC of all necessary files add the shared
-            # libraries to the binaries.
-            self.binaries.append((self.tcl_lib[0], self.tcl_lib[1], 'BINARY'))
-            self.binaries.append((self.tk_lib[0], self.tk_lib[1], 'BINARY'))
-
-            # Only add the intersection of the required and the collected resources, or add all entries if full_tk is
-            # true.
-            self.binaries.extend(toc for toc in tcltk_tree if toc[0] in self.splash_requirements)
-
-        # Handle extra requirements of Tcl/Tk shared libraries (e.g., vcruntime140.dll on Windows - see issue #6284).
-        # These need to be added to splash requirements, so they are extracted into the initial runtime directory in
-        # order to make onefile builds work.
-        #
-        # The really proper way to implement this would be to perform full dependency analysis on self.tcl_lib[0] and
-        # self.tk_lib[0], and ensure that those dependencies are collected and added to splash requirements. This
-        # would, for example, ensure that on Linux, dependent X libraries are collected, just as if the frozen app
-        # itself was using tkinter. On the other hand, collecting all the extra shared libraries on Linux is currently
-        # futile anyway, because the bootloader's parent process would need to set LD_LIBRARY_PATH to the initial
-        # runtime directory to actually have them loaded (and that requires process to be restarted to take effect).
-        #
-        # So for now, we only deal with this on Windows, in a quick'n'dirty work-around way, by assuming that
-        # vcruntime140.dll is already collected as dependency of some other shared library (e.g., the python shared
-        # library).
-        if is_win or is_cygwin:
-            EXTRA_REQUIREMENTS = {'vcruntime140.dll'}
-            self.splash_requirements.update([name for name, *_ in binaries if name.lower() in EXTRA_REQUIREMENTS])
+            self.binaries.extend(entry for entry in tcltk_tree if entry[0] in self.splash_requirements)
 
         # Check if all requirements were found.
-        fnames = [toc[0] for toc in (binaries + datas + self.binaries)]
+        collected_files = set(entry[0] for entry in (binaries + datas + self.binaries))
 
-        def _filter(_item):
-            if _item not in fnames:
+        def _filter_requirement(filename):
+            if filename not in collected_files:
                 # Item is not bundled, so warn the user about it. This actually may happen on some tkinter installations
                 # that are missing the license.terms file.
                 logger.warning(
                     "The local Tcl/Tk installation is missing the file %s. The behavior of the splash screen is "
-                    "therefore undefined and may be unsupported." % _item
+                    "therefore undefined and may be unsupported.", filename
                 )
                 return False
             return True
 
         # Remove all files which were not found.
-        self.splash_requirements = set(filter(_filter, self.splash_requirements))
+        self.splash_requirements = set(filter(_filter_requirement, self.splash_requirements))
 
         # Test if the tcl/tk version is supported by the bootloader.
         self.test_tk_version()
 
-        logger.debug("Splash Requirements: %s" % self.splash_requirements)
+        logger.debug("Splash Requirements: %s", self.splash_requirements)
 
         self.__postinit__()
 
@@ -299,7 +290,7 @@ class Splash(Target):
         return False
 
     def assemble(self):
-        logger.info("Building Splash %s" % self.name)
+        logger.info("Building Splash %s", self.name)
 
         # Function to resize a given image to fit into the area defined by max_img_size.
         def _resize_image(_image, _orig_size):
@@ -330,16 +321,14 @@ class Splash(Target):
                 _img.close()
                 _img_resized.close()
                 _image_data = _image_stream.getvalue()
-                logger.info(
-                    "Resized image %s from dimensions %s to (%d, %d)" % (self.image_file, str(_orig_size), _w, _h)
-                )
+                logger.info("Resized image %s from dimensions %s to (%d, %d)", self.image_file, str(_orig_size), _w, _h)
                 return _image_data
             else:
                 raise ValueError(
                     "The splash image dimensions (w: %d, h: %d) exceed max_img_size (w: %d, h:%d), but the image "
                     "cannot be resized due to missing PIL.Image! Either install the Pillow package, adjust the "
-                    "max_img_size, or use an image of compatible dimensions." %
-                    (_orig_size[0], _orig_size[1], self.max_img_size[0], self.max_img_size[1])
+                    "max_img_size, or use an image of compatible dimensions.", _orig_size[0], _orig_size[1],
+                    self.max_img_size[0], self.max_img_size[1]
                 )
 
         # Open image file
@@ -367,11 +356,11 @@ class Splash(Target):
                 img.save(image_data, format='PNG')
                 img.close()
                 image = image_data.getvalue()
-            logger.info("Converted image %s to PNG format" % self.image_file)
+            logger.info("Converted image %s to PNG format", self.image_file)
         else:
             raise ValueError(
                 "The image %s needs to be converted to a PNG file, but PIL.Image is not available! Either install the "
-                "Pillow package, or use a PNG image for you splash screen." % self.image_file
+                "Pillow package, or use a PNG image for you splash screen.", self.image_file
             )
 
         image_file.close()
@@ -395,15 +384,15 @@ class Splash(Target):
         if tcl_version < 8.6 or tk_version < 8.6:
             logger.warning(
                 "The installed Tcl/Tk (%s/%s) version might not work with the splash screen feature of the bootloader. "
-                "The bootloader is tested against Tcl/Tk 8.6" %
-                (self._tkinter_module.TCL_VERSION, self._tkinter_module.TK_VERSION)
+                "The bootloader is tested against Tcl/Tk 8.6", self._tkinter_module.TCL_VERSION,
+                self._tkinter_module.TK_VERSION
             )
 
         # This should be impossible, since tcl/tk is released together with the same version number, but just in case
         if tcl_version != tk_version:
             logger.warning(
                 "The installed version of Tcl (%s) and Tk (%s) do not match. PyInstaller is tested against matching "
-                "versions" % (self._tkinter_module.TCL_VERSION, self._tkinter_module.TK_VERSION)
+                "versions", self._tkinter_module.TCL_VERSION, self._tkinter_module.TK_VERSION
             )
 
         # Ensure that Tcl is built with multi-threading support.
@@ -444,14 +433,19 @@ class Splash(Target):
             script = re.sub(' +', ' ', script)
 
         # Write script to disk, so that it is transparent to the user what script is executed.
-        with open(self.script_name, "w") as script_file:
+        with open(self.script_name, "w", encoding="utf-8") as script_file:
             script_file.write(script)
         return script
 
     @staticmethod
-    def _uses_tkinter(binaries):
-        # Test for _tkinter instead of tkinter, because a user might use a different wrapping library for tk.
-        return '_tkinter' in binaries.filenames
+    def _uses_tkinter(tkinter_file, binaries):
+        # Test for _tkinter extension instead of tkinter module, because user might use a different wrapping library for
+        # Tk. Use `pathlib.PurePath˙ in comparisons to account for case normalization and separator normalization.
+        tkinter_file = pathlib.PurePath(tkinter_file)
+        for dest_name, src_name, typecode in binaries:
+            if pathlib.PurePath(src_name) == tkinter_file:
+                return True
+        return False
 
     @staticmethod
     def _find_rundir(structure):
