@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2022, PyInstaller Development Team.
+# Copyright (c) 2005-2023, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -43,7 +43,7 @@ from copy import deepcopy
 
 from PyInstaller import HOMEPATH, PACKAGEPATH
 from PyInstaller import log as logging
-from PyInstaller.building.datastruct import TOC
+from PyInstaller.building.utils import add_suffix_to_extension
 from PyInstaller.compat import (
     BAD_MODULE_TYPES, BINARY_MODULE_TYPES, MODULE_TYPES_TO_TOC_DICT, PURE_PYTHON_MODULE_TYPES, PY3_BASE_MODULES,
     VALID_MODULE_TYPES, importlib_load_source, is_win
@@ -52,7 +52,7 @@ from PyInstaller.depend import bytecode
 from PyInstaller.depend.imphook import AdditionalFilesCache, ModuleHookCache
 from PyInstaller.depend.imphookapi import (PreFindModulePathAPI, PreSafeImportModuleAPI)
 from PyInstaller.lib.modulegraph.find_modules import get_implies
-from PyInstaller.lib.modulegraph.modulegraph import ModuleGraph, DEFAULT_IMPORT_LEVEL
+from PyInstaller.lib.modulegraph.modulegraph import ModuleGraph, DEFAULT_IMPORT_LEVEL, ABSOLUTE_IMPORT_LEVEL, Package
 from PyInstaller.log import DEBUG, INFO, TRACE
 from PyInstaller.utils.hooks import collect_submodules, is_package
 
@@ -366,18 +366,74 @@ class PyiModuleGraph(ModuleGraph):
             # For example, we want the excluded imports specified by hook for PIL to be also applied when the referring
             # module is its submodule, PIL.Image.
             excluded_imports = self._find_all_excluded_imports(source_module.identifier)
-            target_module_parts = target_module_partname.split('.')
-            # An excluded import that specifies a package needs to recursively applied to all that package's children.
-            for excluded_import in excluded_imports:
-                excluded_import_parts = excluded_import.split('.')
-                match = target_module_parts[:len(excluded_import_parts)] == excluded_import_parts
-                if match:
+
+            # Apply extra processing only if we have any excluded-imports rules
+            if excluded_imports:
+                # Resolve the base module name. Level can be ABSOLUTE_IMPORT_LEVEL (= 0) for absolute imports, or an
+                # integer indicating the relative level. We do not use equality comparison just in case we ever happen
+                # to get ABSOLUTE_OR_RELATIVE_IMPORT_LEVEL (-1), which is a remnant of python2 days.
+                if level > ABSOLUTE_IMPORT_LEVEL:
+                    if isinstance(source_module, Package):
+                        # Package
+                        base_module_name = source_module.identifier
+                    else:
+                        # Module in a package; base name must be the parent package name!
+                        base_module_name = '.'.join(source_module.identifier.split('.')[:-1])
+
+                    if target_module_partname:
+                        base_module_name += '.' + target_module_partname
+
+                    # Adjust the base module name based on level
+                    if level > 1:
+                        base_module_name = '.'.join(base_module_name.split('.')[:-(level - 1)])
+                else:
+                    base_module_name = target_module_partname
+
+                def _exclude_module(module_name, excluded_imports):
+                    """
+                    Helper for checking whether given module should be excluded.
+                    Returns the name of exclusion rule if module should be excluded, None otherwise.
+                    """
+                    module_name_parts = module_name.split('.')
+                    for excluded_import in excluded_imports:
+                        excluded_import_parts = excluded_import.split('.')
+                        match = module_name_parts[:len(excluded_import_parts)] == excluded_import_parts
+                        if match:
+                            return excluded_import
+                    return None
+
+                # First, check if base module name is to be excluded.
+                # This covers both basic `import a` and `import a.b.c`, as well as `from d import e, f` where base
+                # module `d` is excluded.
+                excluded_import_rule = _exclude_module(base_module_name, excluded_imports)
+                if excluded_import_rule:
                     logger.debug(
                         "Suppressing import of %r from module %r due to excluded import %r specified in a hook for %r "
-                        "(or its parent package(s)).", target_module_partname, source_module.identifier,
-                        excluded_import, source_module.identifier
+                        "(or its parent package(s)).", base_module_name, source_module.identifier, excluded_import_rule,
+                        source_module.identifier
                     )
                     return []
+
+                # If we have target attribute names, check each of them, and remove excluded ones from the
+                # `target_attr_names` list.
+                if target_attr_names:
+                    filtered_target_attr_names = []
+                    for target_attr_name in target_attr_names:
+                        submodule_name = base_module_name + '.' + target_attr_name
+                        excluded_import_rule = _exclude_module(submodule_name, excluded_imports)
+                        if excluded_import_rule:
+                            logger.debug(
+                                "Suppressing import of %r from module %r due to excluded import %r specified in a hook "
+                                "for %r (or its parent package(s)).", submodule_name, source_module.identifier,
+                                excluded_import_rule, source_module.identifier
+                            )
+                        else:
+                            filtered_target_attr_names.append(target_attr_name)
+
+                    # Swap with filtered target attribute names list; if no elements remain after the filtering, pass
+                    # None...
+                    target_attr_names = filtered_target_attr_names or None
+
         return super()._safe_import_hook(target_module_partname, source_module, target_attr_names, level, edge_attr)
 
     def _safe_import_module(self, module_basename, module_name, parent_package):
@@ -489,9 +545,9 @@ class PyiModuleGraph(ModuleGraph):
                     code_dict[node.identifier] = node.code
         return code_dict
 
-    def _make_toc(self, typecode=None, existing_TOC=None):
+    def _make_toc(self, typecode=None):
         """
-        Return the name, path and type of selected nodes as a TOC, or appended to a TOC. The selection is via a list
+        Return the name, path and type of selected nodes as a TOC. The selection is determined by the given list
         of PyInstaller TOC typecodes. If that list is empty we return the complete flattened graph as a TOC with the
         ModuleGraph note types in place of typecodes -- meant for debugging only. Normally we return ModuleGraph
         nodes whose types map to the requested PyInstaller typecode(s) as indicated in the MODULE_TYPES_TO_TOC_DICT.
@@ -507,16 +563,17 @@ class PyiModuleGraph(ModuleGraph):
         regex_str = '(' + '|'.join(PY3_BASE_MODULES) + r')(\.|$)'
         module_filter = re.compile(regex_str)
 
-        result = existing_TOC or TOC()
+        toc = list()
         for node in self.iter_graph(start=self._top_script_node):
             # Skip modules that are in base_library.zip.
             if module_filter.match(node.identifier):
                 continue
             entry = self._node_to_toc(node, typecode)
+            # Append the entry. We do not check for duplicates here; the TOC normalization is left to caller.
+            # However, as entries are obtained from modulegraph, there should not be any duplicates at this stage.
             if entry is not None:
-                # TOC.append the data. This checks for a pre-existing name and skips it if it exists.
-                result.append(entry)
-        return result
+                toc.append(entry)
+        return toc
 
     def make_pure_toc(self):
         """
@@ -525,11 +582,11 @@ class PyiModuleGraph(ModuleGraph):
         # PyInstaller should handle special module types without code object.
         return self._make_toc(PURE_PYTHON_MODULE_TYPES)
 
-    def make_binaries_toc(self, existing_toc):
+    def make_binaries_toc(self):
         """
         Return all binary Python modules formatted as TOC.
         """
-        return self._make_toc(BINARY_MODULE_TYPES, existing_toc)
+        return self._make_toc(BINARY_MODULE_TYPES)
 
     def make_missing_toc(self):
         """
@@ -547,7 +604,7 @@ class PyiModuleGraph(ModuleGraph):
         mg_type = type(node).__name__
         assert mg_type is not None
 
-        if typecode and not (mg_type in typecode):
+        if typecode and mg_type not in typecode:
             # Type is not a to be selected one, skip this one
             return None
         # Extract the identifier and a path if any.
@@ -572,16 +629,13 @@ class PyiModuleGraph(ModuleGraph):
         toc_type = MODULE_TYPES_TO_TOC_DICT[mg_type]
         return name, path, toc_type
 
-    def nodes_to_toc(self, node_list, existing_TOC=None):
+    def nodes_to_toc(self, nodes):
         """
         Given a list of nodes, create a TOC representing those nodes. This is mainly used to initialize a TOC of
         scripts with the ones that are runtime hooks. The process is almost the same as _make_toc(), but the caller
         guarantees the nodes are valid, so minimal checking.
         """
-        result = existing_TOC or TOC()
-        for node in node_list:
-            result.append(self._node_to_toc(node))
-        return result
+        return [self._node_to_toc(node) for node in nodes]
 
     # Return true if the named item is in the graph as a BuiltinModule node. The passed name is a basename.
     def is_a_builtin(self, name):
@@ -755,9 +809,8 @@ class PyiModuleGraph(ModuleGraph):
         require metadata for some distribution (which may not be its own) at runtime. In the case of a match,
         collect the required metadata.
         """
-        from pkg_resources import DistributionNotFound
-
         from PyInstaller.utils.hooks import copy_metadata
+        from PyInstaller.compat import importlib_metadata
 
         # Generate sets of possible function names to search for.
         need_metadata = set()
@@ -782,7 +835,7 @@ class PyiModuleGraph(ModuleGraph):
                         elif function_name in need_recursive_metadata:
                             out.update(copy_metadata(package, recursive=True))
 
-                    except DistributionNotFound:
+                    except importlib_metadata.PackageNotFoundError:
                         # Currently, we opt to silently skip over missing metadata.
                         continue
 
@@ -792,10 +845,36 @@ class PyiModuleGraph(ModuleGraph):
         """
         Return the list of collected python packages.
         """
+        # `node.identifier` might be an instance of `modulegraph.Alias`, hence explicit conversion to `str`.
         return [
-            node.identifier for node in self.iter_graph(start=self._top_script_node) if type(node).__name__ == 'Package'
+            str(node.identifier) for node in self.iter_graph(start=self._top_script_node)
+            if type(node).__name__ == 'Package'
         ]
 
+    def make_hook_binaries_toc(self) -> list:
+        """
+        Return the TOC list of binaries collected by hooks."
+        """
+        toc = []
+        for node in self.iter_graph(start=self._top_script_node):
+            module_name = str(node.identifier)
+            for dest_name, src_name in self._additional_files_cache.binaries(module_name):
+                toc.append((dest_name, src_name, 'BINARY'))
+
+        return toc
+
+    def make_hook_datas_toc(self) -> list:
+        """
+        Return the TOC list of data files collected by hooks."
+        """
+        toc = []
+        for node in self.iter_graph(start=self._top_script_node):
+            module_name = str(node.identifier)
+            for dest_name, src_name in self._additional_files_cache.datas(module_name):
+                toc.append((dest_name, src_name, 'DATA'))
+
+        return toc
+
 
 _cached_module_graph_ = None
 
@@ -824,6 +903,14 @@ def initialize_modgraph(excludes=(), user_hook_dirs=()):
     user_hook_dirs = user_hook_dirs or ()
     excludes = excludes or ()
 
+    # Ensure that __main__ is always excluded from the modulegraph, to prevent accidentally pulling PyInstaller itself
+    # into the modulegraph. This seems to happen on Windows, because modulegraph is able to resolve `__main__` as
+    # `.../PyInstaller.exe/__main__.py` and analyze it. The `__main__` has a different meaning during analysis compared
+    # to the program run-time, when it refers to the program's entry-point (which would always be part of the
+    # modulegraph anyway, by virtue of being the starting point of the analysis).
+    if "__main__" not in excludes:
+        excludes += ("__main__",)
+
     # If there is a graph cached with the same excludes, reuse it. See ``PyiModulegraph._reset()`` for what is
     # reset. This cache is used primarily to speed up the test-suite. Fixture `pyi_modgraph` calls this function with
     # empty excludes, creating a graph suitable for the huge majority of tests.
@@ -867,7 +954,7 @@ def get_bootstrap_modules():
     # Import 'struct' modules to get real paths to module file names.
     mod_struct = __import__('struct')
     # Basic modules necessary for the bootstrap process.
-    loader_mods = TOC()
+    loader_mods = list()
     loaderpath = os.path.join(HOMEPATH, 'PyInstaller', 'loader')
     # On some platforms (Windows, Debian/Ubuntu) '_struct' and zlib modules are built-in modules (linked statically)
     # and thus does not have attribute __file__. 'struct' module is required for reading Python bytecode from
@@ -879,7 +966,7 @@ def get_bootstrap_modules():
             if os.path.basename(os.path.dirname(mod_file)) == 'lib-dynload':
                 # Divert extensions originating from python's lib-dynload directory, to match behavior of #5604.
                 mod_name = os.path.join('lib-dynload', mod_name)
-            loader_mods.append((mod_name, mod_file, 'EXTENSION'))
+            loader_mods.append(add_suffix_to_extension(mod_name, mod_file, 'EXTENSION'))
     loader_mods.append(('struct', os.path.abspath(mod_struct.__file__), 'PYMODULE'))
     # Loader/bootstrap modules.
     # NOTE: These modules should be kept simple without any complicated dependencies.
