@@ -483,6 +483,30 @@ cleanup:
     return rc;
 }
 
+/* Check if the TOC entry corresponds to extractable file */
+static bool
+_pyi_arch_is_extractable(const TOC *ptoc)
+{
+    switch (ptoc->typcd) {
+        /* onefile mode */
+        case ARCHIVE_ITEM_BINARY:
+        case ARCHIVE_ITEM_DATA:
+        case ARCHIVE_ITEM_ZIPFILE:
+        case ARCHIVE_ITEM_SYMLINK: {
+            return true;
+        }
+        /* MERGE mode */
+        case ARCHIVE_ITEM_DEPENDENCY: {
+            return true;
+        }
+        default: {
+            break;
+        }
+    }
+
+    return false;
+}
+
 /* Setup the archive with python modules and the paths required by rest of
  * this module (this always needs to be done).
  * Sets f_archivename, f_homepath, f_mainpath
@@ -490,6 +514,8 @@ cleanup:
 bool
 pyi_arch_setup(ARCHIVE_STATUS *status, char const *archive_path, char const *executable_path)
 {
+    const TOC *ptoc;
+
     /* Copy archive path and executable path */
     if (snprintf(status->archivename, PATH_MAX, "%s", archive_path) >= PATH_MAX) {
         return false;
@@ -503,6 +529,18 @@ pyi_arch_setup(ARCHIVE_STATUS *status, char const *archive_path, char const *exe
         return false;
     }
 
+    /* Check if contents need to be extracted */
+    status->needs_to_extract = false;
+
+    ptoc = status->tocbuff;
+    while (ptoc < status->tocend) {
+        if (_pyi_arch_is_extractable(ptoc)) {
+            status->needs_to_extract = true;
+            break;
+        }
+        ptoc = pyi_arch_increment_toc_ptr(status, ptoc);
+    }
+
     /* Set homepath (a.k.a. sys._MEIPASS) */
     char executable_dir[PATH_MAX];
     pyi_path_dirname(executable_dir, executable_path);
@@ -521,13 +559,13 @@ pyi_arch_setup(ARCHIVE_STATUS *status, char const *archive_path, char const *exe
         pyi_path_join(status->homepath, contents_dir, "Frameworks");
     } else {
         const char *contents_directory = pyi_arch_get_option(status, "pyi-contents-directory");
-        if (!contents_directory) {
-            FATALERROR("pyi-contents-directory option not found in onedir bundle archive!");
-            return false;
+        if (contents_directory && status->needs_to_extract == false) {
+            char root_path[PATH_MAX];
+            pyi_path_dirname(root_path, archive_path);
+            pyi_path_join(status->homepath, root_path, contents_directory);
+        } else {
+            pyi_path_dirname(status->homepath, archive_path);
         }
-        char root_path[PATH_MAX];
-        pyi_path_dirname(root_path, archive_path);
-        pyi_path_join(status->homepath, root_path, contents_directory);
     }
 
     /*
@@ -625,9 +663,23 @@ pyi_arch_find_by_name(const ARCHIVE_STATUS *status, const char *name)
     const TOC *ptoc = status->tocbuff;
 
     while (ptoc < status->tocend) {
+#if defined(_WIN32) || defined(__APPLE__)
+        /* On Windows and macOS, use case-insensitive comparison to
+         * simulate case-insensitive filesystem for extractable entries. */
+        if (_pyi_arch_is_extractable(ptoc)) {
+            if (strcasecmp(ptoc->name, name) == 0) {
+                return ptoc;
+            }
+        } else {
+            if (strcmp(ptoc->name, name) == 0) {
+                return ptoc;
+            }
+        }
+#else
         if (strcmp(ptoc->name, name) == 0) {
             return ptoc;
         }
+#endif
         ptoc = pyi_arch_increment_toc_ptr(status, ptoc);
     }
     return NULL;
