@@ -98,7 +98,6 @@ pyi_main(int argc, char * argv[])
     char archivefile[PATH_MAX];
     int rc = 0;
     int in_child = 0;
-    int needs_to_extract = 0;
     char *extractionpath = NULL;
 
 #ifdef _MSC_VER
@@ -215,8 +214,18 @@ pyi_main(int argc, char * argv[])
 
     /* Check if we need to unpack the embedded archive (onefile build, or onedir
      * build in MERGE mode). If we do, create the temporary directory. */
-    needs_to_extract = pyi_launch_need_to_extract_binaries(archive_status);
-    if (!in_child && needs_to_extract) {
+    if (!in_child && archive_status->needs_to_extract) {
+        /* On Windows, initialize security descriptor for temporary directory.
+         * This is required by `pyi_win32_mkdir()` calls made when creating application's
+         * temporary directory and its sub-directories during file extration. */
+#if defined(_WIN32)
+        VS("LOADER: initializing security descriptor for temporary directory...\n");
+        if (pyi_win32_initialize_security_descriptor() == -1) {
+            FATALERROR("Failed to initialize security descriptor for temporary directory!\n");
+            return -1;
+        }
+#endif
+
         VS("LOADER: creating temporary directory...\n");
         if (pyi_arch_create_tempdir(archive_status) == -1) {
             return -1;
@@ -227,7 +236,7 @@ pyi_main(int argc, char * argv[])
 #if defined(_WIN32) || defined(__APPLE__)
 
     /* On Windows and Mac use single-process for --onedir mode. */
-    if (!extractionpath && !needs_to_extract) {
+    if (!extractionpath && !archive_status->needs_to_extract) {
         VS("LOADER: No need to extract files to run; setting extractionpath to homepath\n");
         extractionpath = archive_status->homepath;
     }
@@ -239,7 +248,7 @@ pyi_main(int argc, char * argv[])
      * set environment (i.e., LD_LIBRARY_PATH) and then restart/replace the
      * process via exec() without fork() for the environment changes (library
      * search path) to take effect. */
-     if (!extractionpath && !needs_to_extract) {
+     if (!extractionpath && !archive_status->needs_to_extract) {
         VS("LOADER: No need to extract files to run; setting up environment and restarting bootloader...\n");
 
         /* Set _MEIPASS2, so that the restarted bootloader process will enter
@@ -252,10 +261,12 @@ pyi_main(int argc, char * argv[])
          * be shown. */
         pyi_setenv("_PYI_ONEDIR_MODE", "1");
 
-        /* Set up the environment, especially LD_LIBRARY_PATH. This is the
-         * main reason we are going to restart the bootloader in the first
-         * place. */
-        if (pyi_utils_set_environment(archive_status) == -1) {
+        /* Set up the library search path (by modifying LD_LIBRARY_PATH or
+         * equivalent), so that the restarted process will be able to find
+         * the collected libraries in the top-level application directory
+         * (i.e., archive_status->homepath).
+         */
+        if (pyi_utils_set_library_search_path(archive_status->homepath) == -1) {
             return -1;
         }
 
@@ -415,12 +426,17 @@ pyi_main(int argc, char * argv[])
             return -1;
         }
 
+        /* At this point, extraction to temporary directory is complete,
+         * and we can free the Windows security descriptor that was used
+         * during creation of temporary directory and its sub-directories. */
+#if defined(_WIN32)
+        pyi_win32_free_security_descriptor();
+#endif
+
         /* Run the 'child' process, then clean up. */
 
         VS("LOADER: Executing self as child\n");
-        pyi_setenv("_MEIPASS2",
-                   archive_status->temppath[0] !=
-                   0 ? archive_status->temppath : archive_status->homepath);
+        pyi_setenv("_MEIPASS2", archive_status->temppath);
 
         VS("LOADER: set _MEIPASS2 to %s\n", pyi_getenv("_MEIPASS2"));
 
@@ -435,9 +451,13 @@ pyi_main(int argc, char * argv[])
 
 #endif  /* defined(__linux__) */
 
-        if (pyi_utils_set_environment(archive_status) == -1) {
+        /* On OSes other than Windows and macOS, we need to set library
+         * search path (via LD_LIBRARY_PATH or equivalent). */
+#if !defined(_WIN32) && !defined(__APPLE__)
+        if (pyi_utils_set_library_search_path(archive_status->temppath) == -1) {
             return -1;
         }
+#endif /* !defined(_WIN32) && !defined(__APPLE__) */
 
         /* Transform parent to background process on OSX only. */
         pyi_parent_to_background();
