@@ -436,7 +436,7 @@ remove_one(wchar_t *wfnm, size_t pos, struct _wfinddata_t wfinfo)
     if (wcscmp(wfinfo.name, L".") == 0  || wcscmp(wfinfo.name, L"..") == 0) {
         return;
     }
-    wfnm[pos] = PYI_NULLCHAR;
+    wfnm[pos] = 0;
     wcscat(wfnm, wfinfo.name);
 
     if ((wfinfo.attrib & _A_SUBDIR)) {
@@ -496,7 +496,7 @@ remove_one(char *pnm, int pos, const char *fnm)
     if (strcmp(fnm, ".") == 0  || strcmp(fnm, "..") == 0) {
         return;
     }
-    pnm[pos] = PYI_NULLCHAR;
+    pnm[pos] = 0;
     strcat(pnm, fnm);
 
     /* Use lstat() instead of stat() to prevent recursion into
@@ -752,12 +752,6 @@ pyi_utils_dlclose(dylib_t dll)
 
 #ifdef _WIN32
 
-int
-pyi_utils_set_environment(const ARCHIVE_STATUS *status)
-{
-    return 0;
-}
-
 static BOOL WINAPI
 _pyi_win32_console_ctrl(DWORD dwCtrlType)
 {
@@ -886,22 +880,24 @@ pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
 
 #else /* ifdef _WIN32 */
 
-static int
-set_dynamic_library_path(const char* path)
+#if !defined(__APPLE__)
+
+int
+pyi_utils_set_library_search_path(const char *path)
 {
     int rc = 0;
     char *env_var, *env_var_orig;
     char *new_path, *orig_path;
 
-    #ifdef AIX
+#ifdef AIX
     /* LIBPATH is used to look up dynamic libraries on AIX. */
     env_var = "LIBPATH";
     env_var_orig = "LIBPATH_ORIG";
-    #else
+#else
     /* LD_LIBRARY_PATH is used on other *nix platforms (except Darwin). */
     env_var = "LD_LIBRARY_PATH";
     env_var_orig = "LD_LIBRARY_PATH_ORIG";
-    #endif /* AIX */
+#endif /* AIX */
 
     /* keep original value in a new env var so the application can restore it
      * before forking subprocesses. This is important so that e.g. a forked
@@ -922,51 +918,7 @@ set_dynamic_library_path(const char* path)
     return rc;
 }
 
-int
-pyi_utils_set_environment(const ARCHIVE_STATUS *status)
-{
-    int rc = 0;
-
-    #ifdef __APPLE__
-    /* On Mac OS X we do not use environment variables DYLD_LIBRARY_PATH
-     * or others to tell OS where to look for dynamic libraries.
-     * There were some issues with this approach. In some cases some
-     * system libraries were trying to load incompatible libraries from
-     * the dist directory. For instance this was experienced with macprots
-     * and PyQt applications.
-     *
-     * To tell the OS where to look for dynamic libraries we modify
-     * .so/.dylib files to use relative paths to other dependent
-     * libraries starting with @executable_path.
-     *
-     * For more information see:
-     * http://blogs.oracle.com/dipol/entry/dynamic_libraries_rpath_and_mac
-     * http://developer.apple.com/library/mac/#documentation/DeveloperTools/  \
-     *     Conceptual/DynamicLibraries/100-Articles/DynamicLibraryUsageGuidelines.html
-     */
-    /* For environment variable details see 'man dyld'. */
-    pyi_unsetenv("DYLD_FRAMEWORK_PATH");
-    pyi_unsetenv("DYLD_FALLBACK_FRAMEWORK_PATH");
-    pyi_unsetenv("DYLD_VERSIONED_FRAMEWORK_PATH");
-    pyi_unsetenv("DYLD_LIBRARY_PATH");
-    pyi_unsetenv("DYLD_FALLBACK_LIBRARY_PATH");
-    pyi_unsetenv("DYLD_VERSIONED_LIBRARY_PATH");
-    pyi_unsetenv("DYLD_ROOT_PATH");
-
-    #else
-
-    /* Set library path to temppath. This is only for onefile mode.*/
-    if (status->temppath[0] != PYI_NULLCHAR) {
-        rc = set_dynamic_library_path(status->temppath);
-    }
-    /* Set library path to homepath. This is for default onedir mode.*/
-    else {
-        rc = set_dynamic_library_path(status->homepath);
-    }
-    #endif /* ifdef __APPLE__ */
-
-    return rc;
-}
+#endif /* !defined(__APPLE__) */
 
 /*
  * If the program is activated by a systemd socket, systemd will set
@@ -982,7 +934,7 @@ pyi_utils_set_environment(const ARCHIVE_STATUS *status)
  * So the application can detect it and use the LISTEN_FDS created
  * by systemd.
  */
-int
+static int
 set_systemd_env()
 {
     const char * env_var = "LISTEN_PID";
