@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2021, PyInstaller Development Team.
+# Copyright (c) 2005-2023, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -9,20 +9,14 @@
 # SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
 #-----------------------------------------------------------------------------
 
-
 import sys
 
-
 frozen_encoding = str(sys.getfilesystemencoding())
 
-
 # For various OS is encoding different.
-# On Windows it should be still mbcs up to Python 3.5
 if sys.platform.startswith('win'):
-    encoding = 'mbcs'
-    if sys.version_info >= (3, 6):
-        # See PEP 529 for more information.
-        encoding = 'utf-8'
+    # See PEP 529 for more information.
+    encoding = 'utf-8'
 # On Mac OS X the value should be still the same.
 elif sys.platform.startswith('darwin'):
     encoding = 'utf-8'
@@ -30,11 +24,8 @@ elif sys.platform.startswith('darwin'):
 else:
     encoding = 'utf-8'
 
-
 print('Encoding expected: ' + encoding)
 print('Encoding current: ' + frozen_encoding)
 
-
 if not frozen_encoding == encoding:
-    raise SystemExit('Frozen encoding %s is not the same as unfrozen %s.' %
-                     (frozen_encoding, encoding))
+    raise SystemExit('Frozen encoding %s is not the same as unfrozen %s.' % (frozen_encoding, encoding))
