@@ -176,7 +176,7 @@ def test_module__file__attribute(pyi_builder):
 def test_module_attributes(tmpdir, pyi_builder):
     # Create file in tmpdir with path to python executable and if it is running in debug mode.
     # Test script uses python interpreter to compare module attributes.
-    with open(os.path.join(tmpdir.strpath, 'python_exe.build'), 'w') as f:
+    with open(os.path.join(tmpdir.strpath, 'python_exe.build'), 'w', encoding='utf8') as f:
         f.write(sys.executable + "\n")
         f.write('debug=%s' % __debug__ + '\n')
         # On Windows we need to preserve system PATH for subprocesses in tests.
@@ -369,7 +369,7 @@ def test_stderr_encoding(tmpdir, pyi_builder):
     #             py.test has stdout encoding 'cp1252', which is an ANSI codepage. test fails as they do not match.
     # with -s:    py.test process has stdout encoding from windows terminal, which is an OEM codepage. spawned
     #             subprocess has the same encoding. test passes.
-    with open(os.path.join(tmpdir.strpath, 'stderr_encoding.build'), 'w') as f:
+    with open(os.path.join(tmpdir.strpath, 'stderr_encoding.build'), 'w', encoding='utf-8') as f:
         if sys.stderr.isatty():
             enc = str(sys.stderr.encoding)
         else:
@@ -381,7 +381,7 @@ def test_stderr_encoding(tmpdir, pyi_builder):
 
 
 def test_stdout_encoding(tmpdir, pyi_builder):
-    with open(os.path.join(tmpdir.strpath, 'stdout_encoding.build'), 'w') as f:
+    with open(os.path.join(tmpdir.strpath, 'stdout_encoding.build'), 'w', encoding='utf-8') as f:
         if sys.stdout.isatty():
             enc = str(sys.stdout.encoding)
         else:
@@ -630,7 +630,7 @@ def test_onefile_longpath(pyi_builder, tmpdir):
     # Create data file with secret
     _SECRET = 'LongDataPath'
     src_filename = tmpdir / 'data.txt'
-    with open(src_filename, 'w') as fp:
+    with open(src_filename, 'w', encoding='utf-8') as fp:
         fp.write(_SECRET)
     # Generate long target filename/path; eight equivalents of SHA256 strings plus data.txt should push just the
     # _MEIPASS-relative path beyond 260 characters...
@@ -759,12 +759,12 @@ def test_package_entry_point_name_collision(pyi_builder):
     ]
 
     # Include a verification that unfrozen Python does still work.
-    p = subprocess.run([sys.executable, str(script)], stdout=subprocess.PIPE, universal_newlines=True)
+    p = subprocess.run([sys.executable, str(script)], stdout=subprocess.PIPE, encoding="utf-8")
     assert re.findall("Running (.*) as (.*)", p.stdout) == expected
 
     pyi_builder.test_script(str(script))
     exe, = pyi_builder._find_executables("matching_name")
-    p = subprocess.run([exe], stdout=subprocess.PIPE, universal_newlines=True)
+    p = subprocess.run([exe], stdout=subprocess.PIPE, encoding="utf-8")
     assert re.findall("Running (.*) as (.*)", p.stdout) == expected
 
 
@@ -784,8 +784,26 @@ def test_contents_directory(pyi_builder):
     assert not (bundle / "foo").exists()
     assert (bundle / "é³þ³źć🚀").is_dir()
 
-    with pytest.raises(SystemExit, match='Invalid value "\\." passed'):
-        pyi_builder.test_source("", pyi_args=["--contents-directory=.", "--noconfirm"])
+    with pytest.raises(SystemExit, match='Invalid value "\\.\\." passed'):
+        pyi_builder.test_source("", pyi_args=["--contents-directory=..", "--noconfirm"])
+
+
+def test_legacy_onedir_layout(pyi_builder):
+    """
+    Test the --contents-directory=., which re-enables the legacy onedir layout.
+    """
+    if pyi_builder._mode != 'onedir':
+        pytest.skip('--contents-directory does not affect onefile builds.')
+
+    pyi_builder.test_source(
+        """
+        import sys
+        import os
+        assert sys._MEIPASS == os.path.dirname(sys.executable)
+        assert os.path.dirname(__file__) == os.path.dirname(sys.executable)
+        """,
+        pyi_args=["--contents-directory", "."]
+    )
 
 
 def test_spec_options(pyi_builder, SPEC_DIR, capsys):
@@ -797,7 +815,7 @@ def test_spec_options(pyi_builder, SPEC_DIR, capsys):
         pyi_args=["--", "--optional-dependency", "email", "--optional-dependency", "gzip"]
     )
     exe, = pyi_builder._find_executables("pyi_spec_options")
-    p = subprocess.run([exe], stdout=subprocess.PIPE, text=True)
+    p = subprocess.run([exe], stdout=subprocess.PIPE, encoding="utf-8")
     assert p.stdout == "Available dependencies: email gzip\n"
 
     capsys.readouterr()
