@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2021, PyInstaller Development Team.
+# Copyright (c) 2005-2023, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -11,13 +11,13 @@
 
 from PyInstaller.utils.tests import importorskip
 
+
 @importorskip('pkg_resources')
 def test_pkg_resources_importable(pyi_builder):
     """
     Check that a trivial example using pkg_resources does build.
     """
-    pyi_builder.test_source(
-        """
+    pyi_builder.test_source("""
         import pkg_resources
         pkg_resources.working_set.require()
         """)
