@@ -49,7 +49,7 @@ def test_frozen_stdlib_modules(pyi_builder, script_dir, tmpdir):
 
     def _load_results(filename):
         # Read pprint-ed results
-        with open(filename, 'r') as fp:
+        with open(filename, 'r', encoding='utf-8') as fp:
             data = fp.read()
         data = eval(data)
 
@@ -184,7 +184,7 @@ def test_onefile_cleanup_symlinked_dir(pyi_builder, tmpdir):
     os.mkdir(output_dir)
     for idx in range(5):
         output_file = os.path.join(output_dir, f'preexisting-{idx}.txt')
-        with open(output_file, 'w') as fp:
+        with open(output_file, 'w', encoding='utf-8') as fp:
             fp.write(f'Pre-existing file #{idx}')
 
     # Check if OS supports creation of symbolic links
@@ -206,7 +206,7 @@ def test_onefile_cleanup_symlinked_dir(pyi_builder, tmpdir):
         # Create five files
         for idx in range(5):
             output_file = os.path.join(output_dir, f'output-{idx}.txt')
-            with open(output_file, 'w') as fp:
+            with open(output_file, 'w', encoding='utf-8') as fp:
                 fp.write(f'Output file #{idx}')
         """,
         app_args=[output_dir]
@@ -317,7 +317,7 @@ def test_inspect_rthook_robustness(pyi_builder):
 @pytest.mark.darwin
 def test_bundled_shell_script(pyi_builder, tmpdir):
     script_file = tmpdir / "test_script.sh"
-    with open(script_file, "w") as fp:
+    with open(script_file, "w", encoding="utf-8") as fp:
         print('#!/bin/sh', file=fp)
         print('echo "Hello world!"', file=fp)
     os.chmod(script_file, 0o755)
@@ -335,3 +335,83 @@ def test_bundled_shell_script(pyi_builder, tmpdir):
         """,
         pyi_args=['--add-data', str(script_file) + os.pathsep + '.']
     )
+
+
+# Test that a program importing `__main__` module does not pull in `PyInstaller` (or in the case of the test, the
+# `pytest`). The problem is that the `__main__` has different meaning during analysis vs. during program's run;
+# during analysis, it resolves to the entry-point module that is running the analysis, whereas during program run, it
+# refers to the program's entry-point. Currently, this seems to be a problem only on Windows, where modulegraph manages
+# to resolve `__main__` into `.../PyInstaller.exe/__main__.py` (or `.../pytest.exe/__main__.py`). On Linux and macOS,
+# modulegraph does not seem to be able to resolve `__main__`.
+def test_import_main_should_not_collect_pyinstaller1(pyi_builder):
+    hooks_dir = os.path.join(_MODULES_DIR, 'pyi_import_main', 'hooks')
+    pyi_builder.test_source(
+        """
+        # Plain import.
+        import __main__
+        print(__main__)
+        """,
+        pyi_args=['--additional-hooks-dir', hooks_dir]
+    )
+
+
+def test_import_main_should_not_collect_pyinstaller2(pyi_builder):
+    hooks_dir = os.path.join(_MODULES_DIR, 'pyi_import_main', 'hooks')
+    pyi_builder.test_source(
+        """
+        # Import __main__ in the same way as `pkg_resources` and its vendored variants
+        # (e.g., `pip._vendor.pkg_resources`) do.
+        try:
+            from __main__ import __requires__
+        except ImportError:
+            pass
+        """,
+        pyi_args=['--additional-hooks-dir', hooks_dir]
+    )
+
+
+# Test that a relative import attempt of a missing optional sub-module in a package does not trigger collection of an
+# unrelated but eponymous top-level module. Simulates the scenario from #8010, where the following block in
+# `openpyxl.reader.excel`:
+#
+# ```
+# try:
+#    from ..tests import KEEP_VBA
+# except ImportError:
+#    KEEP_VBA = False
+# ```
+#
+# (https://foss.heptapod.net/openpyxl/openpyxl/-/blob/branch/3.1/openpyxl/reader/excel.py#L16)
+#
+# triggers collection of top-level `tests` package that is provided by the `LaoNLP` distribution. And importing
+# the said `tests` package during analysis triggers LaoNLP's unit tests...
+def test_missing_relative_import_collects_unrelated_top_level_module(pyi_builder):
+    extra_path = os.path.join(_MODULES_DIR, "pyi_missing_relative_import")
+    hooks_dir = os.path.join(extra_path, 'hooks')
+
+    pyi_builder.test_source(
+        """
+        import mypackage
+        """, pyi_args=['--additional-hooks-dir', hooks_dir, '--paths', extra_path]
+    )
+
+
+# Test that various forms of relative imports are properly caught by the module exclusion.
+@pytest.mark.parametrize('exclude', [False, True], ids=["baseline", "exclude"])
+def test_excluded_relative_imports(pyi_builder, exclude):
+    extra_path = os.path.join(_MODULES_DIR, "pyi_excluded_relative_imports")
+    hooks_dir = os.path.join(extra_path, 'hooks')
+
+    pyi_args = ['--paths', extra_path]
+    if exclude:
+        pyi_args += ['--additional-hooks-dir', hooks_dir]
+
+    pyi_builder.test_source(
+        """
+        import os
+        os.environ['_FORBIDDEN_MODULES_ENABLED'] = '{0}'  # '0' or '1'
+
+        import mypackage
+        """.format(str(int(not exclude))),
+        pyi_args=pyi_args,
+    )
